
c calculate luminosity distances and such
c see David Hogg's paper, astro-ph/9905116

      program distcalc

      parameter(maxnum=10001)
      parameter(IAUTOPAR=0)
      real z(maxnum),ez(maxnum)
      real dc(maxnum),dm(maxnum),da(maxnum),dl(maxnum)
      real distmod(maxnum),dvc(maxnum),tlookback(maxnum)
      real vc(maxnum)

      real function arcsinh

      pi = 3.14159265
      fourpi = 4.0*pi

      nmax=maxnum
      if (IAUTOPAR .eq. 1) then
c set params automatically
c hubble constant      
         h = 0.7
c univ type: omegam, omegal 
c 1:(1,0), 2:(0.05,0), 3(0.2,0.8), 4(0.3,0.7)
         iuniverse = 4
c omega_mass = 8pi G rho_0 / 3H_0**2, omega_lambda = Lambda c**2 / 3H_0**2
         if (iuniverse .eq. 1) then
            omegam = 1.0
            omegal = 0.0
         else if (iuniverse .eq. 2) then
            omegam = 0.05
            omegal = 0.0
         else if (iuniverse .eq. 3) then
            omegam = 0.2
            omegal = 0.8
         else 
            omegam = 0.3
            omegal = 0.7
         end if
      else
         write(*,'("h, omega_m, omega_lambda: ",$)')
         read (*,*) h,omegam,omegal
      end if
      omegak = 1.0 - omegam - omegal

      ckms = 3.0e5
      h0kms = 100.*h
      h0sec = h0kms / 3.086e18
c Hubble distance in Mpc
      dhmpc = ckms / h0kms
c Hubble time in sec
      thsec = 1.0 / h0sec
      thgyr = thsec / (3600.0 * 24.0 * 365.0) / 1.0e9
      write(*,'(a,4(2x,f5.2))') "h, omegam, omegal, omegak: ",
     $     h,omegam,omegal,omegak
      write(*,'(a,1pe9.3,2x,f6.2,2x,1pe9.3)') "thsec, thgyr, dhmpc: ",
     $     thsec, thgyr, dhmpc
      dz = 0.001
      zmax= 10.0001
      z0 = 0.0
      nzmax = int((zmax-z0)/dz) + 1
      write(*,*) "dz, nzmax: ",dz, nzmax
      if (nzmax .gt. nmax) then
         write(*,*) "Too small dz / too small array"
         stop
      end if
      do i=1,nzmax
         z(i) = z0+i*dz
      end do

c  comoving line-of-sight distance Dc = Dh * integral(0,z, dz'/E(z'))
c  Bogus not-really-integrating for the moment

      ez0 = escale(0.0,omegam,omegal,omegak)
      ez(1) = escale(z(i),omegam,omegal,omegak)
      dc(1) = dhmpc * dz * (1./ez(1) + 1./ez0) / 2.
      tlookback(1) = thgyr * dz * ( 1./(1.+z(1))/ez(1) + 1./ez0) / 2.
      do i=2,nzmax
         ez(i) = escale(z(i),omegam,omegal,omegak)
         dc(i) = dc(i-1) + dhmpc * dz * (1./ez(i) + 1./ez(i-1)) / 2.
         tlookback(i) = tlookback(i-1) + thgyr * dz *
     $       ( 1./(1.+z(i))/ez(i) + 1./(1.+z(i-1))/ez(i-1)) / 2.
      end do

      do i=1,nzmax
c  transverse comoving distance
         if (abs(omegak) .lt. 1.e-4) then
            dm(i) = dc(i)
         else if (omegak .gt. 0.) then
            dm(i) = dhmpc / sqrt(omegak) * 
     $           sinh(sqrt(omegak)*dc(i)/dhmpc)
         else
            dm(i) = dhmpc / sqrt(abs(omegak)) * 
     $           sin(sqrt(abs(omegak))*dc(i)/dhmpc)
         end if
c  angular diameter distance
         da(i) = dm(i) / (1.+z(i))
c  luminosity distance
         dl(i) = dm(i) * (1.+z(i))
c  distance modulus.  dl(i) is in Mpc and magnitudes are referred to 10 pc.
         distmod(i) = 5. * log10(dl(i)*1.0e5)
c  comoving volume element, one needs to multiply this by
c  dOmega dz (i.e. solid angle * dz) to get dV_c
         dvc(i) = dhmpc*(1.+z(i))**2*da(i)**2/ez(i)
c actual integrated comoving volume, out to redshift z, over
c the whole sky (so multiply by dOmega/4pi for a given solid angle)
         if (abs(omegak) .lt. 1.e-4) then
            vc(i) = fourpi/3. * dm(i)**3
         else 
            tmp1 = fourpi*dhmpc**3 / 2. / omegak
            tmp2 = dm(i)/dhmpc * sqrt(1.+omegak*(dm(i)/dhmpc)**2)
            tmp3 = sqrt(abs(omegak)) * dm(i) / dhmpc
            if (omegak .gt. 0.) then
               vc(i) = tmp1*(tmp2 - 1./sqrt(abs(omegak))*arcsinh(tmp3))
            else
               vc(i) = tmp1*(tmp2 - 1./sqrt(abs(omegak))*asin(tmp3))
            end if
         end if        
      end do

      write(*,'("dz interval for print out: ",$)')
      read (*,*) dzprint
      nprint = nint(dzprint/dz)
      nprint = max(nprint,1)

      open(2,file="distcalc.out",status='unknown')
c      do i=1,nzmax
      do i=nprint,nzmax,nprint
         write(2,1000) z(i),dc(i),dm(i),da(i),dl(i),distmod(i),
     $        dvc(i),vc(i),tlookback(i)
      end do
 1000 format(f7.4,4(2x,f8.1),2x,f6.2,2(2x,1pe10.4),2x,f8.4)
      close(2)

      end


c----------
c  calculate function E(z)

      function escale(z,omegam,omegal,omegak)
      esq = omegam*(1.+z)**3 + omegal + omegak*(1.+z)**2
      escale=sqrt(esq)
      return
      end

c----------
      function arcsinh(x)

      if (x .ge. 0.0) then
         arcsinh = log(x + sqrt(1. + x**2))
      else
         y=-x
         arcsinh = - log(y + sqrt(1. + y**2))
      end if
      return
      end

         
